@@ -79,7 +79,10 @@ enum class BufferUsage: GLenum {
     StreamCopy = GL_STREAM_COPY,
     #endif
 
-    /** Set once by the application and used frequently for drawing. */
+    /**
+     * Set once by the application and used frequently for drawing. A good
+     * default choice if you are not sure.
+     */
     StaticDraw = GL_STATIC_DRAW,
 
     #ifndef MAGNUM_TARGET_GLES2
@@ -1017,16 +1020,16 @@ class MAGNUM_GL_EXPORT Buffer: public AbstractObject {
          *      @fn_gl_extension_keyword{NamedBufferData,EXT,direct_state_access},
          *      eventually @fn_gl{BindBuffer} and @fn_gl_keyword{BufferData}
          */
-        Buffer& setData(Containers::ArrayView<const void> data, BufferUsage usage);
+        Buffer& setData(Containers::ArrayView<const void> data, BufferUsage usage = BufferUsage::StaticDraw);
 
         /** @overload */
-        template<class T> Buffer& setData(const std::vector<T>& data, BufferUsage usage) {
+        template<class T> Buffer& setData(const std::vector<T>& data, BufferUsage usage = BufferUsage::StaticDraw) {
             setData({data.data(), data.size()}, usage);
             return *this;
         }
 
         /** @overload */
-        template<std::size_t size, class T> Buffer& setData(const std::array<T, size>& data, BufferUsage usage) {
+        template<std::size_t size, class T> Buffer& setData(const std::array<T, size>& data, BufferUsage usage = BufferUsage::StaticDraw) {
             setData({data.data(), data.size()}, usage);
             return *this;
         }
@@ -1164,7 +1167,7 @@ class MAGNUM_GL_EXPORT Buffer: public AbstractObject {
          *      instead.
          */
         template<class T> CORRADE_DEPRECATED("use non-templated map() and Containers::arrayCast() instead") T* map(GLintptr offset, GLsizeiptr length, MapFlags flags) {
-            return Containers::arrayCast<T>(map(offset, length, flags));
+            return reinterpret_cast<T*>(map(offset, length, flags).data());
         }
         #endif
 
