@@ -189,6 +189,18 @@ template<UnsignedInt dimensions> class Texture: public AbstractTexture {
          */
         explicit Texture(NoCreateT) noexcept: AbstractTexture{NoCreate, Implementation::textureTarget<dimensions>()} {}
 
+        /** @brief Copying is not allowed */
+        Texture(const Texture<dimensions>&) = delete;
+
+        /** @brief Move constructor */
+        Texture(Texture<dimensions>&&) noexcept = default;
+
+        /** @brief Copying is not allowed */
+        Texture<dimensions>& operator=(const Texture<dimensions>&) = delete;
+
+        /** @brief Move assignment */
+        Texture<dimensions>& operator=(Texture<dimensions>&&) noexcept = default;
+
         #if !defined(MAGNUM_TARGET_GLES2) && !defined(MAGNUM_TARGET_WEBGL)
         /**
          * @brief Bind level of texture to given image unit
@@ -584,12 +596,21 @@ template<UnsignedInt dimensions> class Texture: public AbstractTexture {
          *      @gl_extension{EXT,sRGB} and
          *      @gl_extension{ANDROID,extension_pack_es31a} /
          *      @gl_extension2{EXT,texture_sRGB_decode,texture_sRGB_decode}
-         * @requires_gles SRGB decode is not available in WebGL.
+         * @requires_gles sRGB decode is not available in WebGL.
          */
-        Texture<dimensions>& setSRGBDecode(bool decode) {
-            AbstractTexture::setSRGBDecode(decode);
+        Texture<dimensions>& setSrgbDecode(bool decode) {
+            AbstractTexture::setSrgbDecode(decode);
             return *this;
         }
+
+        #ifdef MAGNUM_BUILD_DEPRECATED
+        /** @brief @copybrief setSrgbDecode()
+         * @deprecated Use @ref setSrgbDecode() instead.
+         */
+        CORRADE_DEPRECATED("use setSrgbDecode() instead") Texture<dimensions>& setSRGBDecode(bool decode) {
+            return setSrgbDecode(decode);
+        }
+        #endif
         #endif
 
         #if !defined(MAGNUM_TARGET_GLES2) && !defined(MAGNUM_TARGET_WEBGL)
