@@ -43,17 +43,16 @@ namespace Magnum { namespace Shaders {
 @brief Vertex color shader
 
 Draws vertex-colored mesh. You need to provide @ref Position and @ref Color
-attributes in your triangle mesh and call at least
-@ref setTransformationProjectionMatrix().
+attributes in your triangle mesh. By default, the shader renders the mesh in
+an identity transformation. Use @ref setTransformationProjectionMatrix() to
+configure the shader.
 
-@image html shaders-vertexcolor.png
+@image html shaders-vertexcolor.png width=256px
 
 @section Shaders-VertexColor-example Example usage
 
-Common mesh setup. Note the explicit specification of components for the color
-attribute --- the shader accepts four-component color attribute but, similarly
-to all other attributes, it's possible to supply also three-component colors if
-alpha is not important.
+Common mesh setup. The shader accepts either three- or four-component color
+attribute, use either @ref Color3 or @ref Color4 to specify which one you use.
 
 @snippet MagnumShaders.cpp VertexColor-usage1
 
@@ -68,19 +67,35 @@ template<UnsignedInt dimensions> class MAGNUM_SHADERS_EXPORT VertexColor: public
         /**
          * @brief Vertex position
          *
-         * @ref shaders-generic "Generic attribute", @ref Vector2 in 2D,
-         * @ref Vector3 in 3D.
+         * @ref shaders-generic "Generic attribute",
+         * @ref Magnum::Vector2 "Vector2" in 2D @ref Magnum::Vector3 "Vector3"
+         * in 3D.
          */
         typedef typename Generic<dimensions>::Position Position;
 
         /**
-         * @brief Vertex color
+         * @brief Three-component vertex color.
          *
-         * @ref shaders-generic "Generic attribute", @ref Color4, however
-         * defaults to @ref Color3 if @ref MAGNUM_BUILD_DEPRECATED is defined.
-         * See the @ref Generic::Color for more information.
+         * @ref shaders-generic "Generic attribute", @ref Magnum::Color3. Use
+         * either this or the @ref Color4 attribute.
          */
-        typedef typename Generic<dimensions>::Color Color;
+        typedef typename Generic<dimensions>::Color3 Color3;
+
+        /**
+         * @brief Four-component vertex color.
+         *
+         * @ref shaders-generic "Generic attribute", @ref Magnum::Color4. Use
+         * either this or the @ref Color3 attribute.
+         */
+        typedef typename Generic<dimensions>::Color4 Color4;
+
+        #ifdef MAGNUM_BUILD_DEPRECATED
+        /**
+         * @brief Vertex color
+         * @deprecated Use @ref Color3 or @ref Color4 instead.
+         */
+        typedef CORRADE_DEPRECATED("use Color3 or Color4 instead") typename Generic<dimensions>::Color Color;
+        #endif
 
         explicit VertexColor();
 
@@ -96,11 +111,23 @@ template<UnsignedInt dimensions> class MAGNUM_SHADERS_EXPORT VertexColor: public
          */
         explicit VertexColor(NoCreateT) noexcept: AbstractShaderProgram{NoCreate} {}
 
+        /** @brief Copying is not allowed */
+        VertexColor(const VertexColor<dimensions>&) = delete;
+
+        /** @brief Move constructor */
+        VertexColor(VertexColor<dimensions>&&) noexcept = default;
+
+        /** @brief Copying is not allowed */
+        VertexColor<dimensions>& operator=(const VertexColor<dimensions>&) = delete;
+
+        /** @brief Move assignment */
+        VertexColor<dimensions>& operator=(VertexColor<dimensions>&&) noexcept = default;
+
         /**
          * @brief Set transformation and projection matrix
          * @return Reference to self (for method chaining)
          *
-         * Default is identity matrix.
+         * Default is an identity matrix.
          */
         VertexColor<dimensions>& setTransformationProjectionMatrix(const MatrixTypeFor<dimensions, Float>& matrix) {
             setUniform(_transformationProjectionMatrixUniform, matrix);
